{-
 *  Programmer:	Piotr Borek
 *  E-mail:     piotrborek@op.pl
 *  Copyright 2016 Piotr Borek
 *
 *  Distributed under the terms of the GPL (GNU Public License)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-}

module Coin.UI.Utils.Observable (
    Observable,
    observableNew,
    observableRegister,
    observableSet
) where

import Control.Monad
import Control.Concurrent.MVar

data Observable a = Observable {
    observableList :: MVar [a -> IO ()]
}

observableNew :: IO (Observable a)
observableNew = do
    var <- newMVar []
    return $ Observable var

observableRegister :: Observable a -> (a -> IO ()) -> IO ()
observableRegister observable action =
    modifyMVar_ (observableList observable) $ \actions ->
        return $ action : actions

observableSet :: Observable a -> a -> IO ()
observableSet observable value = do
    actions <- readMVar $ observableList observable
    forM_ actions $ \action -> action value
